"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.approveOperator = exports.calculateClaims = exports.calculateOwnership = exports.getAllCertificateEvents = exports.decodeEvent = exports.getAllOwnedCertificates = exports.getAllCertificates = exports.transferCertificates = exports.claimCertificates = exports.decodeClaimData = exports.encodeClaimData = void 0;
const ethers_1 = require("ethers");
const Certificate_1 = require("./Certificate");
const events_1 = require("../utils/events");
const encodeClaimData = async (claimData, blockchainProperties) => {
    const { beneficiary, address, region, zipCode, countryCode } = claimData;
    return blockchainProperties.issuer.encodeClaimData(beneficiary !== null && beneficiary !== void 0 ? beneficiary : '', address !== null && address !== void 0 ? address : '', region !== null && region !== void 0 ? region : '', zipCode !== null && zipCode !== void 0 ? zipCode : '', countryCode !== null && countryCode !== void 0 ? countryCode : '');
};
exports.encodeClaimData = encodeClaimData;
const decodeClaimData = async (encodedClaimData, blockchainProperties) => {
    const { _beneficiary, _address, _region, _zipCode, _countryCode } = await blockchainProperties.issuer.decodeClaimData(encodedClaimData);
    return {
        beneficiary: _beneficiary,
        address: _address,
        region: _region,
        zipCode: _zipCode,
        countryCode: _countryCode
    };
};
exports.decodeClaimData = decodeClaimData;
async function claimCertificates(certificateIds, claimData, blockchainProperties, forAddress) {
    const certificatesPromises = certificateIds.map((certId) => new Certificate_1.Certificate(certId, blockchainProperties).sync());
    const certificates = await Promise.all(certificatesPromises);
    const { activeUser, registry } = blockchainProperties;
    const claimer = forAddress !== null && forAddress !== void 0 ? forAddress : (await activeUser.getAddress());
    const values = certificates.map((cert) => { var _a; return ethers_1.BigNumber.from((_a = cert.owners[claimer]) !== null && _a !== void 0 ? _a : 0); });
    const encodedClaimData = await exports.encodeClaimData(claimData, blockchainProperties);
    const data = ethers_1.utils.randomBytes(32);
    const registryWithSigner = registry.connect(activeUser);
    const claimTx = await registryWithSigner.safeBatchTransferAndClaimFrom(claimer, claimer, certificateIds, values, data, certificates.map(() => encodedClaimData));
    await claimTx.wait();
    return claimTx;
}
exports.claimCertificates = claimCertificates;
async function transferCertificates(certificateIds, to, blockchainProperties, from) {
    const certificatesPromises = certificateIds.map((certId) => new Certificate_1.Certificate(certId, blockchainProperties).sync());
    const { registry, activeUser } = blockchainProperties;
    const registryWithSigner = registry.connect(activeUser);
    const activeUserAddress = await activeUser.getAddress();
    const fromAddress = from !== null && from !== void 0 ? from : activeUserAddress;
    const certificates = await Promise.all(certificatesPromises);
    const values = certificates.map((cert) => { var _a; return ethers_1.BigNumber.from((_a = cert.owners[fromAddress]) !== null && _a !== void 0 ? _a : 0); });
    const transferTx = await registryWithSigner.safeBatchTransferFrom(fromAddress, to, certificateIds, values, ethers_1.utils.randomBytes(32) // TO-DO: replace with proper data
    );
    await transferTx.wait();
    return transferTx;
}
exports.transferCertificates = transferCertificates;
async function getAllCertificates(blockchainProperties) {
    const { issuer } = blockchainProperties;
    const certificationRequestApprovedEvents = await events_1.getEventsFromContract(issuer, issuer.filters.CertificationRequestApproved(null, null, null));
    const certificatePromises = certificationRequestApprovedEvents.map((event) => new Certificate_1.Certificate(event._certificateId.toNumber(), blockchainProperties).sync());
    return Promise.all(certificatePromises);
}
exports.getAllCertificates = getAllCertificates;
async function getAllOwnedCertificates(blockchainProperties) {
    const { registry, activeUser } = blockchainProperties;
    const owner = await activeUser.getAddress();
    const transfers = await events_1.getEventsFromContract(registry, registry.filters.TransferSingle(null, null, owner, null, null));
    const certificateIds = [
        ...new Set(transfers.map((transfer) => transfer._id.toNumber()))
    ];
    const balances = await registry.balanceOfBatch(Array(certificateIds.length).fill(owner), certificateIds);
    const available = certificateIds.filter((id, index) => !balances[index].isZero());
    const certificatePromises = available.map((id) => new Certificate_1.Certificate(id, blockchainProperties).sync());
    return Promise.all(certificatePromises);
}
exports.getAllOwnedCertificates = getAllOwnedCertificates;
async function decodeEvent(eventName, event, contract) {
    const eventBlock = await contract.provider.getBlock(event.blockHash);
    return {
        name: eventName,
        transactionHash: event.transactionHash,
        blockHash: event.blockHash,
        timestamp: eventBlock.timestamp,
        ...contract.interface.decodeEventLog(eventName, event.data, event.topics)
    };
}
exports.decodeEvent = decodeEvent;
const getAllCertificateEvents = async (certId, blockchainProperties) => {
    const { registry } = blockchainProperties;
    const getEvent = async (filter, eventName) => {
        const logs = await registry.provider.getLogs({
            ...filter,
            fromBlock: 0,
            toBlock: 'latest'
        });
        const parsedLogs = await Promise.all(logs.map(async (event) => decodeEvent(eventName, event, registry)));
        return parsedLogs.filter((event) => event._id.toNumber() === certId);
    };
    const issuanceSingleEvents = await getEvent(registry.filters.IssuanceSingle(null, null, null), 'IssuanceSingle');
    const transferSingleEvents = await getEvent(registry.filters.TransferSingle(null, null, null, null, null), 'TransferSingle');
    const claimSingleEvents = await getEvent(registry.filters.ClaimSingle(null, null, null, null, null, null), 'ClaimSingle');
    return [...issuanceSingleEvents, ...transferSingleEvents, ...claimSingleEvents];
};
exports.getAllCertificateEvents = getAllCertificateEvents;
const calculateOwnership = async (certificateId, blockchainProperties) => {
    const ownedShares = {};
    const { registry } = blockchainProperties;
    const transferSingleEvents = (await events_1.getEventsFromContract(registry, registry.filters.TransferSingle(null, null, null, null, null))).filter((event) => event._id.eq(certificateId));
    const transferBatchEvents = (await events_1.getEventsFromContract(registry, registry.filters.TransferBatch(null, null, null, null, null))).filter((e) => e._ids.some((id) => id.eq(certificateId)));
    const allHistoricOwners = [
        ...new Set([...transferSingleEvents, ...transferBatchEvents].map((event) => event._to))
    ];
    const allHistoricOwnersBalances = await Promise.all(allHistoricOwners.map((ownerAddress) => registry.balanceOf(ownerAddress, certificateId)));
    allHistoricOwners.forEach((owner, index) => {
        ownedShares[owner] = allHistoricOwnersBalances[index].toString();
    });
    return ownedShares;
};
exports.calculateOwnership = calculateOwnership;
const calculateClaims = async (certificateId, blockchainProperties) => {
    const claimedShares = {};
    const { registry } = blockchainProperties;
    const claimSingleEvents = (await events_1.getEventsFromContract(registry, registry.filters.ClaimSingle(null, null, null, null, null, null))).filter((event) => event._id.eq(certificateId));
    const claimBatchEvents = (await events_1.getEventsFromContract(registry, registry.filters.ClaimBatch(null, null, null, null, null, null))).filter((e) => e._ids.some((id) => id.eq(certificateId)));
    const allHistoricClaimers = [
        ...new Set([...claimSingleEvents, ...claimBatchEvents].map((event) => event._claimSubject))
    ];
    const allHistoricClaimersBalances = await Promise.all(allHistoricClaimers.map((claimerAddress) => registry.claimedBalanceOf(claimerAddress, certificateId)));
    allHistoricClaimers.forEach((owner, index) => {
        claimedShares[owner] = allHistoricClaimersBalances[index].toString();
    });
    return claimedShares;
};
exports.calculateClaims = calculateClaims;
async function approveOperator(operator, blockchainProperties) {
    const { activeUser, registry } = blockchainProperties;
    const registryWithSigner = registry.connect(activeUser);
    return registryWithSigner.setApprovalForAll(operator, true);
}
exports.approveOperator = approveOperator;
//# sourceMappingURL=CertificateUtils.js.map